@@ -6,6 +6,7 @@ import logging
 import os
 import re
 import uuid
+import warnings
 
 import numpy as np
 import pandas as pd
@@ -53,11 +54,6 @@ def load_datatables(skip_if_already_loaded=True):
         return
 
     load_datatables_js = read_package_file("javascript", "load_datatables_connected.js")
-    eval_functions_js = read_package_file("javascript", "eval_functions.js")
-    load_datatables_js += (
-        "\n$('head').append(`<script>\n" + eval_functions_js + "\n</` + 'script>');"
-    )
-
     display(Javascript(load_datatables_js))
 
     _DATATABLE_LOADED = True
@@ -85,6 +81,14 @@ def _formatted_values(df):
     return formatted_df.values.tolist()
 
 
+def replace_value(template, pattern, value, count=1):
+    """Set the given pattern to the desired value in the template,
+    after making sure that the pattern is found exactly once."""
+    assert isinstance(template, str)
+    assert template.count(pattern) == count
+    return template.replace(pattern, value)
+
+
 def _datatables_repr_(df=None, tableId=None, **kwargs):
     """Return the HTML/javascript representation of the table"""
 
@@ -99,6 +103,7 @@ def _datatables_repr_(df=None, tableId=None, **kwargs):
     maxBytes = kwargs.pop("maxBytes", 0)
     maxRows = kwargs.pop("maxRows", 0)
     maxColumns = kwargs.pop("maxColumns", pd.get_option("display.max_columns") or 0)
+    eval_functions = kwargs.pop("eval_functions", None)
 
     if isinstance(df, (np.ndarray, np.generic)):
         df = pd.DataFrame(df)
@@ -112,6 +117,9 @@ def _datatables_repr_(df=None, tableId=None, **kwargs):
     if "paging" not in kwargs and len(df.index) <= kwargs.get("lengthMenu", [10])[0]:
         kwargs["paging"] = False
 
+    # Load the HTML template
+    output = read_package_file("datatables_template.html")
+
     tableId = tableId or str(uuid.uuid4())
     if isinstance(classes, list):
         classes = " ".join(classes)
@@ -128,43 +136,62 @@ def _datatables_repr_(df=None, tableId=None, **kwargs):
     thead = match.groups()[0]
     if not showIndex:
         thead = thead.replace("<th></th>", "", 1)
-    html_table = (
-        '<table id="'
-        + tableId
-        + '" class="'
-        + classes
-        + '"><thead>'
-        + thead
-        + "</thead></table>"
+    table_header = (
+        f'<table id="{tableId}" class="{classes}"><thead>{thead}</thead></table>'
     )
+    output = replace_value(
+        output,
+        '<table id="table_id"><thead><tr><th>A</th></tr></thead></table>',
+        table_header,
+    )
+    output = replace_value(output, "#table_id", f"#{tableId}", count=2)
 
-    kwargs["data"] = _formatted_values(df.reset_index() if showIndex else df)
+    # Export the DT args to JSON
+    dt_args = json.dumps(kwargs)
 
-    try:
-        dt_args = json.dumps(kwargs)
-        return (
-            """<div>"""
-            + html_table
-            + """
-<script type="text/javascript">
-require(["datatables"], function (datatables) {
-    $(document).ready(function () {
-        var dt_args = """
-            + dt_args
-            + """;
-        dt_args = eval_functions(dt_args);
-        table = $('#"""
-            + tableId
-            + """').DataTable(dt_args);
-    });
-})
-</script>
-</div>
-"""
+    # And load the eval_functions_js library if required
+    if eval_functions:
+        eval_functions_js = read_package_file("javascript", "eval_functions.js")
+        output = replace_value(
+            output,
+            "// eval_functions_js",
+            f"<script>\n{eval_functions_js}\n<script>",
         )
-    except TypeError as error:
-        logger.error(str(error))
-        return ""
+        output = replace_value(
+            output,
+            "let dt_args = {};",
+            f"let dt_args = eval_functions({dt_args});",
+        )
+    else:
+        output = replace_value(output, "let dt_args = {};", f"let dt_args = {dt_args};")
+        if eval_functions is None and _any_function(kwargs):
+            warnings.warn(
+                "One of the arguments passed to datatables starts with 'function'. "
+                "To evaluate this function, use the option 'eval_functions=True'. "
+                "To silence this warning, use 'eval_functions=False'."
+            )
+
+    # Export the table data to JSON and include this in the HTML
+    data = _formatted_values(df.reset_index() if showIndex else df)
+    dt_data = json.dumps(data)
+    output = replace_value(output, "const data = [];", f"const data = {dt_data};")
+
+    return output
+
+
+def _any_function(value):
+    """Does a value or nested value starts with 'function'?"""
+    if isinstance(value, str) and value.startswith("function"):
+        return True
+    elif isinstance(value, list):
+        for nested_value in value:
+            if _any_function(nested_value):
+                return True
+    elif isinstance(value, dict):
+        for key, nested_value in value.items():
+            if _any_function(nested_value):
+                return True
+    return False
 
 
 def show(df=None, **kwargs):
